require 'win32ole'
require_relative 'connection_factory.rb'
require_relative 'errors.rb'

class OICFiscalPrinterLib
  attr_accessor :connection
  
  def initialize(args)
    @connection = WIN32OLE.new('OICFiscalPrinterLib.OICFiscalPrinter')

    @connection.PortConfigString = ConnectionFactory.new(args).port_config_string
    
    @connection.PrinterDriver = args[:printerdriver] || default_printerdriver

    @connection.LogTransmission = args.fetch(:logtransmission, default_logtransmission) 
    @connection.LogFolderName   = args[:logfoldername] || default_logfoldername

    initialize_extra(args)
  end

  def initialize_extra(args)
  end  


  def method_missing(method_name, *args)
    is_property = false
    if method_name.match(/=$/)
      method_name = method_name.to_s.gsub(/=$/,'')
      is_property = true
    else
      method_name = method_name.to_s
    end    

    if respond_to?(method_name)
      if is_property
        connection.setproperty(method_name, *args)
      else  
        connection.send(method_name, *args) #if ready?
      end
    else
      raise NoMethodError, "undefined method `#{method_name}' for OICFiscalPrinterLib interface"
    end  
  end

  def respond_to?(method_name)
    @connection.ole_respond_to?(method_name)
  end

  protected
    def default_logtransmission
      false
    end 

    def default_logfoldername
      'log'
    end

    def default_printerdriver
      'HD E PL 1.0'
    end

    def ready?
      result = false
      result = connection.ReadDLEStatus(nil, nil, nil)
      unless result
        raise ConnectionError, "#{connection.lasterror}: #{connection.lasterrormessage}"
      end
      result 
    end  
end
